@@ -1,24 +1,21 @@
 /*
- * Copyright (c) 2002 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2002-2003 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
+ * The contents of this file constitute Original Code as defined in and
+ * are subject to the Apple Public Source License Version 1.1 (the
+ * "License").  You may not use this file except in compliance with the
+ * License.  Please obtain a copy of the License at
+ * http://www.apple.com/publicsource and read it before using this file.
  * 
- * This file contains Original Code and/or Modifications of Original Code
- * as defined in and that are subject to the Apple Public Source License
- * Version 2.0 (the 'License'). You may not use this file except in
- * compliance with the License. Please obtain a copy of the License at
- * http://www.opensource.apple.com/apsl/ and read it before using this
- * file.
- * 
- * The Original Code and all software distributed under the License are
- * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This Original Code and all software distributed under the License are
+ * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
- * Please see the License for the specific language governing rights and
- * limitations under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
+ * License for the specific language governing rights and limitations
+ * under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -177,7 +174,7 @@ main(argc, argv)
 		protoprivs = getprivs(protoid, quotatype);
 #ifdef __APPLE__
 		if (protoprivs == (struct quotause *) NULL)
-		  exit(0);
+			exit(0);
 #endif /* __APPLE__ */
 		for (qup = protoprivs; qup; qup = qup->next) {
 			qup->dqblk.dqb_btime = 0;
@@ -200,8 +197,20 @@ main(argc, argv)
 #endif /* __APPLE__ */
 		if (writetimes(protoprivs, tmpfd, quotatype) == 0)
 			exit(1);
-		if (editit(tmpfil) && readtimes(protoprivs, tmpfd))
-			putprivs(0, quotatype, protoprivs);
+		if (editit(tmpfil)) {
+			/*
+			 * Re-open tmpfil to be editor independent.
+			 */
+			close(tmpfd);
+			tmpfd = open(tmpfil, O_RDWR, 0);
+			if (tmpfd < 0) {
+				freeprivs(protoprivs);
+				unlink(tmpfil);
+				exit(1);
+			}
+			if (readtimes(protoprivs, tmpfd))
+				putprivs(0, quotatype, protoprivs);
+		}
 		freeprivs(protoprivs);
 		exit(0);
 	}
@@ -213,10 +222,26 @@ main(argc, argv)
 		if (curprivs == (struct quotause *) NULL)
 		  exit(0);
 #endif /* __APPLE__ */
-		if (writeprivs(curprivs, tmpfd, *argv, quotatype) == 0)
+
+
+		if (writeprivs(curprivs, tmpfd, *argv, quotatype) == 0) {
+			freeprivs(curprivs);
 			continue;
-		if (editit(tmpfil) && readprivs(curprivs, tmpfd))
-			putprivs(id, quotatype, curprivs);
+		}
+		if (editit(tmpfil)) {
+			/*
+			 * Re-open tmpfil to be editor independent.
+			 */
+			close(tmpfd);
+			tmpfd = open(tmpfil, O_RDWR, 0);
+			if (tmpfd < 0) {
+				freeprivs(curprivs);
+				unlink(tmpfil);
+				exit(1);
+			}
+			if (readprivs(curprivs, tmpfd))
+				putprivs(id, quotatype, curprivs);
+		}
 		freeprivs(curprivs);
 	}
 	close(tmpfd);
