@@ -3,22 +3,21 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
- * This file contains Original Code and/or Modifications of Original Code
- * as defined in and that are subject to the Apple Public Source License
- * Version 2.0 (the 'License'). You may not use this file except in
- * compliance with the License. Please obtain a copy of the License at
- * http://www.opensource.apple.com/apsl/ and read it before using this
- * file.
+ * "Portions Copyright (c) 1999 Apple Computer, Inc.  All Rights
+ * Reserved.  This file contains Original Code and/or Modifications of
+ * Original Code as defined in and that are subject to the Apple Public
+ * Source License Version 1.0 (the 'License').  You may not use this file
+ * except in compliance with the License.  Please obtain a copy of the
+ * License at http://www.apple.com/publicsource and read it before using
+ * this file.
  * 
  * The Original Code and all software distributed under the License are
  * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
- * Please see the License for the specific language governing rights and
- * limitations under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
+ * License for the specific language governing rights and limitations
+ * under the License."
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -91,8 +90,12 @@ byte_swap_sbin(struct fs *sb)
 	byte_swap_ints(((int32_t *)&sb->fs_firstfield), 52);
 	byte_swap_int(sb->fs_cgrotor);
 	byte_swap_int(sb->fs_cpc);
-	byte_swap_shorts((int16_t *)sb->fs_opostbl, 16 * 8); 
-	byte_swap_ints((int32_t *)sb->fs_sparecon, 50);
+        byte_swap_shorts((int16_t *)sb->fs_opostbl,
+                         sizeof(sb->fs_opostbl) / sizeof(int16_t));
+        byte_swap_int(sb->fs_avgfilesize);
+        byte_swap_int(sb->fs_avgfpdir);
+        byte_swap_ints((int32_t *)sb->fs_sparecon,
+                       sizeof(sb->fs_sparecon) / sizeof(int32_t));
 	byte_swap_ints((int32_t *)&sb->fs_contigsumsize, 3);
 	byte_swap_longlongs((u_int64_t *)&sb->fs_maxfilesize,3);
 	byte_swap_ints((int32_t *)&sb->fs_state, 6);
@@ -125,8 +128,12 @@ byte_swap_sbout(struct fs *sb)
 	byte_swap_ints(((int32_t *)&sb->fs_firstfield), 52);
 	byte_swap_int(sb->fs_cgrotor);
 	byte_swap_int(sb->fs_cpc);
-	byte_swap_shorts((int16_t *)sb->fs_opostbl, 16 * 8); 
-	byte_swap_ints((int32_t *)sb->fs_sparecon, 50);
+        byte_swap_shorts((int16_t *)sb->fs_opostbl,
+                         sizeof(sb->fs_opostbl) / sizeof(int16_t));
+        byte_swap_int(sb->fs_avgfilesize);
+        byte_swap_int(sb->fs_avgfpdir);
+        byte_swap_ints((int32_t *)sb->fs_sparecon,
+                       sizeof(sb->fs_sparecon) / sizeof(int32_t));
 	byte_swap_ints((int32_t *)&sb->fs_contigsumsize, 3);
 	byte_swap_longlongs((u_int64_t *)&sb->fs_maxfilesize,3);
 	byte_swap_ints((int32_t *)&sb->fs_state, 6);
