@@ -86,6 +86,7 @@ CheckHFS( 	int fsReadRef, int fsWriteRef, int checkLevel, int repairLevel,
 	OSErr				err = noErr;
 	OSErr				scavError = 0;
 	int					scanCount = 0;
+	int					isJournaled = 0;
 	Boolean 			autoRepair;
 
 	autoRepair = (fsWriteRef != -1 && repairLevel != kNeverRepair);
@@ -122,7 +123,8 @@ DoAgain:
 		goto termScav;
 	}
 
-	if (CheckIfJournaled(&dataArea)
+	isJournaled = CheckIfJournaled( &dataArea );
+	if (isJournaled != 0
 	    && scanCount == 0
 	    && checkLevel != kForceCheck
 	    && !(checkLevel == kPartialCheck && repairLevel == kForceRepairs)) {
@@ -145,20 +147,40 @@ DoAgain:
 	if (scavError == noErr && logLevel >= kDebugLog)
 		printVerifyStatus(&dataArea);
 
-	if (scavError == noErr && !dataArea.cleanUnmount && fsWriteRef != -1)
+	// Looped for maximum times for verify and repair.  This was the last verify and
+	// we bail out if problems were found
+	if (scanCount >= kMaxReScan && (dataArea.RepLevel != repairLevelNoProblemsFound)) {
+		PrintStatus(&dataArea, M_ReRepairFailed, 1, dataArea.volumeName);
+		scavError = R_RFail;
+		goto termScav;
+	}
+
+	// mark the volume clean if there are no errors and we have write access
+	// and either the volume is not marked as clean or if the volume is journaled.
+	// 
+	if ( scavError == noErr && fsWriteRef != -1 &&
+		 (dataArea.cleanUnmount == false || isJournaled != 0) ) 
 		CheckForClean(&dataArea, true);		/* mark volume clean */
 
-	if ( dataArea.RepLevel == repairLevelUnrepairable )
+	if ( dataArea.RepLevel == repairLevelUnrepairable ) 
 		err = cdUnrepairableErr;
 
 	if ( !autoRepair &&
 	     (dataArea.RepLevel == repairLevelVolumeRecoverable ||
 	      dataArea.RepLevel == repairLevelCatalogBtreeRebuild  ||
-	      dataArea.RepLevel == repairLevelUnrepairable) )
+	      dataArea.RepLevel == repairLevelUnrepairable) ) {
 		PrintStatus(&dataArea, M_NeedsRepair, 1, dataArea.volumeName);
+		scavError = R_VFail;
+		goto termScav;
+	}
 
-	if ( scavError == noErr && dataArea.RepLevel == repairLevelNoProblemsFound )
-		PrintStatus(&dataArea, M_AllOK, 1, dataArea.volumeName);
+	if ( scavError == noErr && dataArea.RepLevel == repairLevelNoProblemsFound ) {
+		if (scanCount == 0) {
+			PrintStatus(&dataArea, M_AllOK, 1, dataArea.volumeName);
+		} else {
+			PrintStatus(&dataArea, M_RepairOK, 1, dataArea.volumeName);
+		}
+	}
 
 	//
 	//	Repair the volume if it needs repairs, its repairable and we were able to unmount it
@@ -187,18 +209,16 @@ DoAgain:
 		if ( scavError == noErr )
 		{
 			*modified = 1;	/* Report back that we made repairs */
-			if ( (dataArea.scanAgain || dataArea.RepLevel == repairLevelCatalogBtreeRebuild) && 
-				  scanCount++ == 0 )
-			{
-				ScavCtrl( &dataArea, scavTerminate, &temp );
-				repairLevel = kMajorRepairs;
-				checkLevel = kAlwaysCheck;
-				PrintStatus(&dataArea, M_Rescan, 0);
-				goto DoAgain;
-			}
-			PrintStatus(&dataArea, M_RepairOK, 1, dataArea.volumeName);
+
+			/* we just repaired a volume, so scan it again to check if it corrected everything properly */
+			ScavCtrl( &dataArea, scavTerminate, &temp );
+			repairLevel = kMajorRepairs;
+			checkLevel = kAlwaysCheck;
+			PrintStatus(&dataArea, M_Rescan, 0);
+			scanCount++;
+			goto DoAgain;
 		}
-		else
+		else 
 			PrintStatus(&dataArea, M_RepairFailed, 1, dataArea.volumeName);
 	}
 	else if ( scavError != noErr ) {
@@ -225,7 +245,9 @@ DoAgain:
 	}
 
 termScav:
-	err = scavError;
+	if (err == noErr) {
+		err = scavError;
+	}
 		
 	//
 	//	Terminate the scavenger
@@ -286,29 +308,38 @@ void ScavCtrl( SGlobPtr GPtr, UInt32 ScavOp, short *ScavRes )
 	switch ( ScavOp )
 	{
 		case scavInitialize:								//	INITIAL VOLUME CHECK
-			if ( result = ScavSetUp( GPtr ) )			//	set up BEFORE CheckForStop
+		{
+			int clean;
+
+			if ( ( result = ScavSetUp( GPtr ) ) )			//	set up BEFORE CheckForStop
 				break;
 			if ( IsBlueBoxSharedDrive( GPtr->DrvPtr ) )
 				break;
-			if ( result = CheckForStop( GPtr ) )		//	in order to initialize wrCnt
+			if ( ( result = CheckForStop( GPtr ) ) )		//	in order to initialize wrCnt
 				break;
-			
-			if (GPtr->chkLevel == kNeverCheck || GPtr->chkLevel == kDirtyCheck) {
-				int clean;
-
-				clean = CheckForClean(GPtr, false);
-				if (clean == 1)
+		
+			/* Call for all chkLevel options and check return value only 
+			 * for kDirtyCheck for preen option and kNeverCheck for quick option
+			 */
+			clean = CheckForClean(GPtr, false);
+			if ((GPtr->chkLevel == kDirtyCheck) || (GPtr->chkLevel == kNeverCheck)) {
+				if (clean == 1) {
+					/* volume was unmounted cleanly */
 					GPtr->cleanUnmount = true;
+					break;
+				}
 
 				if (GPtr->chkLevel == kNeverCheck) {
 					if (clean == -1)
 						result = R_BadSig;
 					else if (clean == 0) {
 						/*
-						 * We lie for journaled file systems since
-						 * they get cleaned up in mount by replaying
-						 * the journal.
-						 */
+					 	 * We lie for journaled file systems since
+					     * they get cleaned up in mount by replaying
+					 	 * the journal.
+					 	 * Note: CheckIfJournaled will return negative
+					     * if it finds lastMountedVersion = FSK!.
+					     */
 						if (CheckIfJournaled(GPtr))
 							GPtr->cleanUnmount = true;
 						else
@@ -316,9 +347,6 @@ void ScavCtrl( SGlobPtr GPtr, UInt32 ScavOp, short *ScavRes )
 					}
 					break;
 				}
-
-				if (GPtr->cleanUnmount)
-					break;
 			}
 			
 			if (CheckIfJournaled(GPtr)
@@ -327,11 +355,14 @@ void ScavCtrl( SGlobPtr GPtr, UInt32 ScavOp, short *ScavRes )
 				&& !(GPtr->chkLevel == kAlwaysCheck && GPtr->repairLevel == kMajorRepairs)) {
 			    break;
 			}
-			
+
 			result = IVChk( GPtr );
+			
 			break;
+		}
 	
 		case scavVerify:								//	VERIFY
+		{
 
 #if SHOW_ELAPSED_TIMES
 			gettimeofday( &myStartTime, &zone );
@@ -350,20 +381,20 @@ void ScavCtrl( SGlobPtr GPtr, UInt32 ScavOp, short *ScavRes )
 
 			if ( IsBlueBoxSharedDrive( GPtr->DrvPtr ) )
 				break;
-			if ( result = CheckForStop( GPtr ) )
+			if ( ( result = CheckForStop( GPtr ) ) )
 				break;
 
 #if SHOW_ELAPSED_TIMES
 			gettimeofday( &myStartTime, &zone );
 #endif
 
-			if ( result = CreateExtentsBTreeControlBlock( GPtr ) )	//	Create the calculated BTree structures
+			if ( ( result = CreateExtentsBTreeControlBlock( GPtr ) ) )	//	Create the calculated BTree structures
 				break;
-			if ( result = CreateCatalogBTreeControlBlock( GPtr ) )
+			if ( ( result = CreateCatalogBTreeControlBlock( GPtr ) ) )
 				break;
-			if ( result = CreateAttributesBTreeControlBlock( GPtr ) )
+			if ( ( result = CreateAttributesBTreeControlBlock( GPtr ) ) )
 				break;
-			if ( result = CreateExtendedAllocationsFCB( GPtr ) )
+			if ( ( result = CreateExtendedAllocationsFCB( GPtr ) ) )
 				break;
 
 #if SHOW_ELAPSED_TIMES
@@ -378,7 +409,7 @@ void ScavCtrl( SGlobPtr GPtr, UInt32 ScavOp, short *ScavRes )
 			CalculateItemCount( GPtr, &GPtr->itemsToProcess, &GPtr->onePercent );
 			GPtr->itemsProcessed += GPtr->onePercent;	// We do this 4 times as set up in CalculateItemCount() to smooth the scroll
 			
-			if ( result = VLockedChk( GPtr ) )
+			if ( ( result = VLockedChk( GPtr ) ) )
 				break;
 
 			GPtr->itemsProcessed += GPtr->onePercent;	// We do this 4 times as set up in CalculateItemCount() to smooth the scroll
@@ -388,7 +419,7 @@ void ScavCtrl( SGlobPtr GPtr, UInt32 ScavOp, short *ScavRes )
 			gettimeofday( &myStartTime, &zone );
 #endif
 				
-			if (result = ExtBTChk(GPtr))
+			if ((result = ExtBTChk(GPtr)))
 				break;
 
 #if SHOW_ELAPSED_TIMES
@@ -399,14 +430,14 @@ void ScavCtrl( SGlobPtr GPtr, UInt32 ScavOp, short *ScavRes )
 				myElapsedTime.tv_sec, myElapsedTime.tv_usec );
 #endif
 				
-			if (result = CheckForStop(GPtr))
+			if ((result = CheckForStop(GPtr)))
 				break;
 			
 			GPtr->itemsProcessed += GPtr->onePercent;	// We do this 4 times as set up in CalculateItemCount() to smooth the scroll
 
-			if (result = ExtFlChk(GPtr))
+			if ((result = ExtFlChk(GPtr)))
 				break;
-			if (result = CheckForStop(GPtr))
+			if ((result = CheckForStop(GPtr)))
 				break;
 			
 			GPtr->itemsProcessed += GPtr->onePercent;	// We do this 4 times as set up in CalculateItemCount() to smooth the scroll
@@ -427,7 +458,7 @@ void ScavCtrl( SGlobPtr GPtr, UInt32 ScavOp, short *ScavRes )
 				break;
 			}
 			
-			if (result = CheckCatalogBTree(GPtr))
+			if ((result = CheckCatalogBTree(GPtr)))
 				break;
 
 #if SHOW_ELAPSED_TIMES
@@ -438,7 +469,7 @@ void ScavCtrl( SGlobPtr GPtr, UInt32 ScavOp, short *ScavRes )
 				myElapsedTime.tv_sec, myElapsedTime.tv_usec );
 #endif
 
-			if (result = CheckForStop(GPtr))
+			if ((result = CheckForStop(GPtr)))
 				break;
 
 			WriteMsg( GPtr, M_CatHChk, kStatusMessage );
@@ -447,7 +478,7 @@ void ScavCtrl( SGlobPtr GPtr, UInt32 ScavOp, short *ScavRes )
 			gettimeofday( &myStartTime, &zone );
 #endif
 				
-			if (result = CatHChk(GPtr))
+			if ((result = CatHChk(GPtr)))
 				break;
 
 #if SHOW_ELAPSED_TIMES
@@ -458,11 +489,11 @@ void ScavCtrl( SGlobPtr GPtr, UInt32 ScavOp, short *ScavRes )
 				myElapsedTime.tv_sec, myElapsedTime.tv_usec );
 #endif
 
-			if (result = CheckForStop(GPtr))
+			if ((result = CheckForStop(GPtr)))
 				break;
-			if (result = AttrBTChk(GPtr))
+			if ((result = AttrBTChk(GPtr)))
 				break;
-			if (result = CheckForStop(GPtr))
+			if ((result = CheckForStop(GPtr)))
 				break;
 
 			WriteMsg( GPtr, M_VolumeBitMapChk, kStatusMessage );
@@ -471,7 +502,7 @@ void ScavCtrl( SGlobPtr GPtr, UInt32 ScavOp, short *ScavRes )
 			gettimeofday( &myStartTime, &zone );
 #endif
 				
-			if (result = CheckVolumeBitMap(GPtr, false))
+			if ((result = CheckVolumeBitMap(GPtr, false)))
 				break;
 
 #if SHOW_ELAPSED_TIMES
@@ -482,7 +513,7 @@ void ScavCtrl( SGlobPtr GPtr, UInt32 ScavOp, short *ScavRes )
 				myElapsedTime.tv_sec, myElapsedTime.tv_usec );
 #endif 
 
-			if (result = CheckForStop(GPtr))
+			if ((result = CheckForStop(GPtr)))
 				break;
 
 			WriteMsg( GPtr, M_VInfoChk, kStatusMessage );
@@ -491,7 +522,7 @@ void ScavCtrl( SGlobPtr GPtr, UInt32 ScavOp, short *ScavRes )
 			gettimeofday( &myStartTime, &zone );
 #endif
 				
-			if (result = VInfoChk(GPtr))
+			if ((result = VInfoChk(GPtr)))
 				break;
 
 #if SHOW_ELAPSED_TIMES
@@ -503,7 +534,7 @@ void ScavCtrl( SGlobPtr GPtr, UInt32 ScavOp, short *ScavRes )
 #endif
 
 			stat =	GPtr->VIStat  | GPtr->ABTStat | GPtr->EBTStat | GPtr->CBTStat | 
-					GPtr->CatStat;
+					GPtr->CatStat | GPtr->JStat;
 			
 			if ( stat != 0 )
 			{
@@ -512,7 +543,7 @@ void ScavCtrl( SGlobPtr GPtr, UInt32 ScavOp, short *ScavRes )
 					//	2200106, We isolate very minor errors so that if the volume cannot be unmounted
 					//	CheckDisk will just return noErr
 					short minorErrors = (GPtr->CatStat & ~S_LockedDirName)  |
-								GPtr->VIStat | GPtr->ABTStat | GPtr->EBTStat | GPtr->CBTStat;
+								GPtr->VIStat | GPtr->ABTStat | GPtr->EBTStat | GPtr->CBTStat | GPtr->JStat;
 					if ( minorErrors == 0 )
 						GPtr->RepLevel =  repairLevelVeryMinorErrors;
 					else
@@ -526,11 +557,13 @@ void ScavCtrl( SGlobPtr GPtr, UInt32 ScavOp, short *ScavRes )
 			GPtr->itemsProcessed = GPtr->itemsToProcess;
 			result = CheckForStop(GPtr);				//	one last check for modified volume
 			break;
-							
+		}					
+		
 		case scavRepair:									//	REPAIR
+		{
 			if ( IsBlueBoxSharedDrive( GPtr->DrvPtr ) )
 				break;
-			if ( result = CheckForStop(GPtr) )
+			if ( ( result = CheckForStop(GPtr) ) )
 				break;
 			if ( GPtr->CBTStat & S_RebuildBTree )
 				WriteMsg( GPtr, M_RebuildingCatalogBTree, kTitleMessage );
@@ -538,10 +571,13 @@ void ScavCtrl( SGlobPtr GPtr, UInt32 ScavOp, short *ScavRes )
 				WriteMsg( GPtr, M_Repair, kTitleMessage );
 			result = RepairVolume( GPtr );
 			break;
+		}
 		
 		case scavTerminate:									//	CLEANUP AFTER SCAVENGE
+		{
 			result = ScavTerm(GPtr);
 			break;
+		}
 	}													//	end ScavOp switch
 
 
@@ -857,6 +893,10 @@ static int ScavSetUp( SGlob *GPtr)
 	GPtr->CBTStat			= 0;
 	GPtr->CatStat			= 0;
 	GPtr->VeryMinorErrorsStat	= 0;
+	GPtr->JStat			= 0;
+
+	/* Assume that the volume is dirty unmounted */
+	GPtr->cleanUnmount		= false;
 
  	// 
  	// Initialize VolumeObject
@@ -872,11 +912,11 @@ static int ScavSetUp( SGlob *GPtr)
 		}
 		return( R_NoMem );
 	}
-	
+
 	// Convert the security attribute name from utf8 to utf16.  This will
 	// avoid repeated conversion of all extended attributes to compare with
 	// security attribute name
-	(void) utf_decodestr(KAUTH_FILESEC_XATTR, strlen(KAUTH_FILESEC_XATTR), GPtr->securityAttrName, &GPtr->securityAttrLen);
+	(void) utf_decodestr((unsigned char *)KAUTH_FILESEC_XATTR, strlen(KAUTH_FILESEC_XATTR), GPtr->securityAttrName, &GPtr->securityAttrLen);
 
 	return( noErr );
 
