@@ -33,8 +33,6 @@
 	Copyright:	 1996-1999 by Apple Computer, Inc., all rights reserved.
 */
 
-#define hasHFSPlusAPIs 0
-
 
 #include "BTree.h"
 #include "Scavenger.h"
@@ -71,6 +69,12 @@ Public (Exported) Routines:
 					This is a queued HFSDispatch call that does the equivalent of
 					MapFileBlockC, using a parameter block.
 
+	UpdateExtentRecord
+					If the extent record came from the extents file, write out
+					the updated record; otherwise, copy the updated record into
+					the FCB resident extent record.  If the record has no extents,
+					and was in the extents file, then delete the record instead.
+
 ============================================================
 Internal Routines:
 ============================================================
@@ -92,11 +96,6 @@ Internal Routines:
 					for the file.
 	DeallocateFork
 					Deallocate all allocation blocks belonging to a given fork.
-	UpdateExtentRecord
-					If the extent record came from the extents file, write out
-					the updated record; otherwise, copy the updated record into
-					the FCB resident extent record.  If the record has no extents,
-					and was in the extents file, then delete the record instead.
 */
 
 enum
@@ -188,13 +187,6 @@ static OSErr TruncateExtents(
 	Boolean *			recordDeleted);
 #endif
 
-static OSErr UpdateExtentRecord (
-	const SVCB		*vcb,
-	SFCB					*fcb,
-	const HFSPlusExtentKey	*extentFileKey,
-	HFSPlusExtentRecord		extentData,
-	UInt32					extentBTreeHint);
-
 static OSErr MapFileBlockFromFCB(
 	const SVCB		*vcb,
 	const SFCB			*fcb,
@@ -447,10 +439,10 @@ OSErr MapFileBlockC (
 	HFSPlusExtentRecord	foundData;
 	UInt32				foundIndex;
 	UInt32				hint;
-	UInt32				firstFABN;				// file allocation block of first block in found extent
+	UInt32				firstFABN = 0;				// file allocation block of first block in found extent
 	UInt32				nextFABN;				// file allocation block of block after end of found extent
 	UInt32				dataEnd;				// (offset) end of range that is contiguous (in sectors)
-	UInt32				startBlock;				// volume allocation block corresponding to firstFABN
+	UInt32				startBlock = 0;			// volume allocation block corresponding to firstFABN
 	UInt64				temp;
 	
 	
@@ -480,12 +472,7 @@ OSErr MapFileBlockC (
 	//
 	
 	// Get fork's physical size, in sectors
-#if hasHFSPlusAPIs
-	if (fcb->fcbFlags & fcbLargeFileMask)
-		temp = BytesToSectors(fcb->fcbPhysicalSize);
-	else
-#endif
-		temp = fcb->fcbPhysicalSize >> kSectorShift;
+	temp = fcb->fcbPhysicalSize >> kSectorShift;
 	dataEnd = nextFABN * allocBlockSize;		// Assume valid data through end of this extent
 	if (temp < dataEnd)							// Is PEOF shorter?
 		dataEnd = temp;							// Yes, so only map up to PEOF
@@ -809,26 +796,19 @@ OSErr ExtendFileC (
 	//
 	//	Determine the physical EOF in allocation blocks
 	//
-#if hasHFSPlusAPIs
-	if (fcb->fcbFlags & fcbLargeFileMask)
-		eofBlocks = BytesToBlocks(&fcb->fcbPhysicalSize, vcb->vcbBlockSize);
-	else
-#endif
-		eofBlocks = fcb->fcbPhysicalSize / vcb->vcbBlockSize;
+	eofBlocks = fcb->fcbPhysicalSize / vcb->vcbBlockSize;
 
 	//
 	//	Make sure the request won't make the file too big (>=2GB).
-	//	[2350148] Always limit HFS files, even if opened with fcbLargeFileMask.
+	//	[2350148] Always limit HFS files.
 	//		Shouldn't really fail if allOrNothing is false
 	//		Adjust for clump size here?
 	//
-#if hasHFSPlusAPIs
-	if ((fcb->fcbFlags & fcbLargeFileMask) && (vcb->vcbSignature == kHFSPlusSigWord))
+	if ( vcb->vcbSignature == kHFSPlusSigWord )
 	{
-		//	HFS Plus file opened with OpenFork.  Allow it to grow beyond 2GB.
+		//	Allow it to grow beyond 2GB.
 	}
 	else
-#endif
 	{
 		UInt32 maxFileBlocks;	//	max legal EOF, in blocks
 		maxFileBlocks = (kTwoGigSectors-1) / sectorsPerBlock;
@@ -1024,12 +1004,7 @@ ErrorExit:
 Exit:
 	*actualSectorsAdded = blocksAdded * sectorsPerBlock;
 	if (blocksAdded) {
-#if hasHFSPlusAPIs
-		if (fcb->fcbFlags & fcbLargeFileMask)
-			MultiplyUInt32IntoUInt64((UInt64*)&fcb->fcbPhysicalSize, eofBlocks, vcb->vcbBlockSize);
-		else
-#endif
-			fcb->fcbPhysicalSize = eofBlocks * vcb->vcbBlockSize;
+		fcb->fcbPhysicalSize = (UInt64)eofBlocks * (UInt64)vcb->vcbBlockSize;
 		fcb->fcbFlags |= fcbModifiedMask;
 	}
 
@@ -1102,12 +1077,7 @@ OSErr TruncateFileC (
 		forkType = kDataForkType;
 
 	//	Compute number of allocation blocks currently in file
-#if hasHFSPlusAPIs
-	if (fcb->fcbFlags & fcbLargeFileMask)
-		physNumBlocks = BytesToSectors(fcb->fcbPhysicalSize) / sectorsPerBlock;
-	else
-#endif
-		physNumBlocks = fcb->fcbPhysicalSize / vcb->vcbBlockSize;
+	physNumBlocks = fcb->fcbPhysicalSize / vcb->vcbBlockSize;
 	
 	//
 	//	Round newPEOF up to a multiple of the allocation block size.  If new size is
@@ -1127,12 +1097,7 @@ OSErr TruncateFileC (
 	//
 	//	Update FCB's length
 	//
-#if hasHFSPlusAPIs
-	if (fcb->fcbFlags & fcbLargeFileMask)
-		MultiplyUInt32IntoUInt64((UInt64*)&fcb->fcbPhysicalSize, nextBlock, vcb->vcbBlockSize);
-	else
-#endif
-		fcb->fcbPhysicalSize = nextBlock * vcb->vcbBlockSize;
+	fcb->fcbPhysicalSize = (UInt64)nextBlock * (UInt64)vcb->vcbBlockSize;
 	fcb->fcbFlags |= fcbModifiedMask;
 	
 	//
@@ -1473,7 +1438,7 @@ Exit:
 //				(other) = error from BTree
 //
 
-static OSErr UpdateExtentRecord (
+OSErr UpdateExtentRecord (
 	const SVCB		*vcb,
 	SFCB					*fcb,
 	const HFSPlusExtentKey	*extentFileKey,
@@ -1716,121 +1681,6 @@ static OSErr MapFileBlockFromFCB(
 }
 
 
-#if 0
-void AdjustEOF(SFCB *sourceFCB)
-{
-	OSErr		err;
-	VCB		*vcb;			//	sourceFCB's volume
-	SFCB		*destFCB;		//	A matching FCB
-	short		refnum;			//	Used for iterating over FCBs
-	Boolean		isHFSPlus;		//	True if sourceFCB is on an HFS Plus volume
-	UInt8		resourceFlag;	//	Contains fcbResourceMask bit from fcbFlags
-	SInt32		pinnedEOF;
-	SInt32		pinnedPhysEOF;
-#if hasHFSPlusAPIs
-	UInt64		wideEOF;
-	UInt64		widePhysEOF;
-#endif
-
-	vcb = sourceFCB->fcbVolume;
-	sourceFCB->fcbFlags |= fcbModifiedMask;
-	resourceFlag = sourceFCB->fcbFlags & fcbResourceMask;
-	
-	isHFSPlus = (vcb->vcbSignature == kHFSPlusSigWord);
-
-#if hasHFSPlusAPIs	
-	//	Determine the EOF and physical EOF values to propagate.
-	if (sourceFCB->fcbFlags & fcbLargeFileMask) {
-		UInt32		maxPinnedSize;
-		maxPinnedSize = (0x7FFFFFFF / vcb->vcbAlBlkSiz) * vcb->vcbAlBlkSiz;
-
-		wideEOF = sourceFCB->fcbLogicalSize;
-		widePhysEOF = sourceFCB->fcbPhysicalSize;
-		
-		// pin endOfFile
-		if (wideEOF > maxPinnedSize)
-			pinnedEOF = maxPinnedSize;
-		else
-			pinnedEOF = sourceFCB->fcbLogicalSize;
-		sourceFCB->fcbEOF = pinnedEOF;
-		
-		// pin physicalEOF
-		if (widePhysEOF > maxPinnedSize)
-			pinnedPhysEOF = maxPinnedSize;
-		else
-			pinnedPhysEOF = sourceFCB->fcbPhysicalSize;
-		sourceFCB->fcbPhysicalSize = pinnedPhysEOF;
-	}
-	else {
-		pinnedEOF = sourceFCB->fcbEOF;
-		pinnedPhysEOF = sourceFCB->fcbPhysicalSize;
-		
-		wideEOF = pinnedEOF;
-		widePhysEOF = pinnedPhysEOF;
-	}
-#else
-	pinnedEOF = sourceFCB->fcbLogicalSize;
-	pinnedPhysEOF = sourceFCB->fcbPhysicalSize;
-#endif
-
-	//	Loop through FCBs for same file.  If same fork, update their sizes and extents.
-	refnum = 0;
-	while (noErr == (err = LocateFCB(vcb, sourceFCB->fcbFileID, &refnum, &destFCB))) {
-		if (destFCB == sourceFCB)
-			continue;					//	skip the source FCB
-		
-		if ((destFCB->fcbFlags & fcbResourceMask) != resourceFlag)
-			continue;					//	skip if different fork
-
-#if hasHFSPlusAPIs		
-		//	We found another FCB for the same fork.  Copy the fields.
-		if (destFCB->fcbFlags & fcbLargeFileMask) {
-			destFCB->fcbLogicalSize = wideEOF;
-			destFCB->fcbPhysicalSize = widePhysEOF;
-		}
-#endif
-		destFCB->fcbLogicalSize = pinnedEOF;
-		destFCB->fcbPhysicalSize = pinnedPhysEOF;
-		destFCB->fcbSBlk = sourceFCB->fcbSBlk;	//	Used by MFS.  May be used by foreign file systems.
-				
-		//	Copy extent info, too
-		if (isHFSPlus) {			
-			CopyMemory(srcFCB->fcbExtents, destFCB->fcbExtents, sizeof(HFSPlusExtentRecord));
-		}
-		else {
-			destFCB->fcbExtRec[0] = sourceFCB->fcbExtRec[0];
-			destFCB->fcbExtRec[1] = sourceFCB->fcbExtRec[1];
-			destFCB->fcbExtRec[2] = sourceFCB->fcbExtRec[2];
-		}
-	}
-}
-#endif
-
-
-#if hasHFSPlusAPIs
-UInt32 BytesToBlocks(const UInt64 *bytes, UInt32 blockSize)
-{
-	UInt32			carry;
-	UnsignedWide	temp;
-	
-	temp = *((UnsignedWide *)bytes);
-
-	//	This loop divides "bytes" by "blockSize".  It assumes blockSize
-	//	is a power of two.  The division is done by shifting both numerator
-	//	and divisor until the divisor is one.
-	
-	while (blockSize > 1) {
-		blockSize >>= 1;
-		carry = (temp.hi & 1) ? 0x80000000 : 0;
-		temp.hi >>= 1;
-		temp.lo = carry | temp.lo >> 1;
-	}
-	
-	return temp.lo;
-}
-#endif
-
-
 //
 //	Routine:	ZeroFileBlocks
 //
@@ -1857,7 +1707,7 @@ OSErr	ZeroFileBlocks( SVCB *vcb, SFCB *fcb, UInt32 startingSector, UInt32 number
 	UInt32					requestedBytes;
 	UInt32					actualBytes;									//	Bytes actually read by CacheReadInPlace
 	UInt32					bufferSizeSectors	= FSBufferSize >> kSectorShift;
-	UInt32					currentPosition		= startingSector << kSectorShift;
+	UInt64					currentPosition		= startingSector << kSectorShift;
 		
 	buffer	= AllocateMemory(FSBufferSize);
 	if ( buffer == NULL )
@@ -1971,7 +1821,7 @@ Boolean NodesAreContiguous(
 	if ( !ExtentsAreIntegral(extents, mask, &blocksChecked, &lastExtentReached) )
 		return false;
 
-	if (lastExtentReached || (blocksChecked * vcb->vcbBlockSize) >= fcb->fcbPhysicalSize)
+	if (lastExtentReached || ((UInt64)blocksChecked * (UInt64)vcb->vcbBlockSize) >= fcb->fcbPhysicalSize)
 		return true;
 
 	startBlock = blocksChecked;
