@@ -35,6 +35,12 @@
 #include "BTreeScanner.h"
 #include "hfs_endian.h"
 
+#include <sys/xattr.h>
+#include <sys/acl.h>
+#include <sys/kauth.h>
+#include <sys/errno.h>
+#include <sys/syslimits.h>
+
 #ifdef __cplusplus
 extern	"C" {
 #endif
@@ -56,11 +62,15 @@ enum {
 // Misc constants
 //
 
+/* IO size for reading or writing disk blocks */
+#define DISK_IOSIZE	32768
+
+#define kMaxReScan	3	/* max times to re-scan volume on repair success */
+
 #define	kBTreeHeaderUserBytes	128
 
 #define	kBusErrorValue	0x50FF8001
 
-
 // Danger! This should not be hard coded
 #define	kMaxClumpSize	0x100000	/* max clump size is 1MB (2048 btree nodes) */
 
@@ -86,6 +96,21 @@ enum {
 #define	VAtrb_Cons	0x0100			/* volume consistency flag */
 #define kHFSCatalogNodeIDsReused 0x1000		
 
+/*
+ *	File type and creator for symbolic links (from xnu/bsd/hfs/hfs.h)
+ */
+enum {
+	kSymLinkFileType  = 0x736C6E6B,	/* 'slnk' */
+	kSymLinkCreator   = 0x72686170	/* 'rhap' */
+};
+
+/*
+ *	File type and creator for TextEdit documents
+ */
+enum {
+	kTextFileType		= 0x54455854,	/* 'TEXT' */
+	kTextFileCreator	= 0x74747874,	/* 'ttxt' */
+};
 
 /*------------------------------------------------------------------------------
  BTree data structures
@@ -139,9 +164,6 @@ typedef	STPR SBTPT[BTMaxDepth]; 		/* BTree path table */
 
 #define fNameLocked 4096
 
-#define kDataFork	0
-#define kRsrcFork	(-1)
-
 union CatalogName {
 	Str31 							pstr;
 	HFSUniStr255 					ustr;
@@ -239,6 +261,10 @@ enum {
 };
 
 
+enum {
+	fsDSIntErr	= -127	/* non-hardware Internal file system error */
+};
+
 //	Repair Info - additional info returned when a repair is attempted
 enum {
 	kFileSharingEnabled		= 0x00000001,
@@ -316,7 +342,7 @@ enum {
 	UInt32		maskBit;	/* incorrect bit */
 	UInt32		hint;		/* B-tree node hint */
 	UInt32		parid;		/* parent ID */
-	unsigned char	name[1];	/* dir or file name */
+	unsigned char name[1];	/* dir or file name */
  } RepairOrder, *RepairOrderPtr;
 
 
@@ -356,15 +382,17 @@ typedef struct MissingThread
 	HFSPlusCatalogThread  thread;
 } MissingThread;
 
-
+#define kDataFork	0
+#define kRsrcFork	(-1)
 
 struct ExtentInfo {
-	HFSCatalogNodeID 				fileNumber;
-	UInt32 							startBlock;
-	UInt32 							blockCount;
-	UInt8							forkType;
-	Boolean							hasThread;
-	UInt16							reservedWord;
+	HFSCatalogNodeID fileID;
+	UInt32	startBlock;
+	UInt32 	blockCount;
+	UInt32	newStartBlock;
+	UInt8	forkType;
+	Boolean	hasThread;
+	Boolean hasRepair;
 };
 typedef struct ExtentInfo ExtentInfo;
 
@@ -430,46 +458,23 @@ union CatalogRecord {
 };
 typedef union CatalogRecord				CatalogRecord;
 
-/*------------------------------------------------------------------------------
- Extended Attributes 
-------------------------------------------------------------------------------*/
-
-/* Name of system security extended attribute (ACL) from sys/kauth.h */ 
-#ifndef KAUTH_FILESEC_XATTR
-#define KAUTH_FILESEC_XATTR "com.apple.system.Security"
-#endif
-
-/* Maximum length of extended attribute from sys/xattr.h */
-#ifndef XATTR_MAXNAMELEN
-#define XATTR_MAXNAMELEN 127
-#endif
-
-/* Catalog file record flags */
-enum {
-	kHFSHasAttributesBit	= 0x0002,	/* object has extended attributes */
-	kHFSHasAttributesMask	= 0x0004,
-
-	kHFSHasSecurityBit	= 0x0003,	/* object has security data (ACLs) */
-	kHFSHasSecurityMask	= 0x0008
-};
-
 /*
   	Key for records in the attributes file.  Fields are compared in the order:
   		cnid, attributeName, startBlock
 */
 
 struct AttributeKey {
-	UInt16 						keyLength;					/* must set kBTBigKeysMask and kBTVariableIndexKeysMask in BTree header's attributes */
-	UInt16 						pad;
+	UInt16 							keyLength;					/* must set kBTBigKeysMask and kBTVariableIndexKeysMask in BTree header's attributes */
+	UInt16 							pad;
 	HFSCatalogNodeID 				cnid;						/* file or folder ID */
-	UInt32 						startBlock;					/* block # relative to start of attribute */
-	UInt16						attrNameLen;					/* number of unicode characters */
-	UInt16						attrName[127];					/* attribute name (Unicode) */
+	UInt32 							startBlock;					/* block # relative to start of attribute */
+	UInt16     attrNameLen;     /* number of unicode characters */
+	UInt16     attrName[127];   /* attribute name (Unicode) */
 };
 typedef struct AttributeKey				AttributeKey;
 enum {
 	kAttributeKeyMaximumLength	= sizeof(AttributeKey) - sizeof(UInt16),
-	kAttributeKeyMinimumLength	= kAttributeKeyMaximumLength - (127 * sizeof(UInt16))
+	kAttributeKeyMinimumLength	= kAttributeKeyMaximumLength - 127 * sizeof(UInt16) + sizeof(UInt16)
 };
 
 struct HIOParam {
@@ -596,7 +601,7 @@ typedef pascal void (*UserMessageProcPtr)(StringPtr message, SInt16 messageType,
 
 #endif
 
-/* 3857929 Structure to detemine consistency of attribute data and 
+/* 3843779 Structure to detemine consistency of attribute data and 
  * corresponding bit in catalog record.  Based on Chinese Remainder
  * Theorem
  */
@@ -666,6 +671,7 @@ typedef struct SGlob {
 	UInt16				CBTStat;				//	scavenge status flags for catalog BTree 
 	UInt16				CatStat;				//	scavenge status flags for catalog file
 	UInt16				VeryMinorErrorsStat;	//	scavenge status flags for very minor errors
+	UInt16				JStat;					//	scavange status flags for journal errors
 	DrvQElPtr			DrvPtr;					//	pointer to driveQ element for target drive
 	UInt32				TarID;					//	target ID (CNID of data structure being verified)
 	UInt64				TarBlock;				//	target block/node number being verified
@@ -725,13 +731,12 @@ typedef struct SGlob {
 	int				logLevel;
 	int				chkLevel;
 	int             repairLevel;
-	Boolean			scanAgain;	//	used to force another scan of the volume (max of 2 scans)
 	Boolean			minorRepairErrors;	// indicates some minor repairs failed
 	int				canWrite;  	// we can safely write to the block device
 	int				lostAndFoundMode;  // used when creating lost+found directory
 	BTScanState		scanState;
 
-	char			volumeName[256]; /* volume name in ASCII or UTF-8 */
+	unsigned char	volumeName[256]; /* volume name in ASCII or UTF-8 */
 
 	PrimeBuckets 	CBTAttrBucket;		/* prime number buckets for Attribute bit in Catalog btree */
 	PrimeBuckets 	CBTSecurityBucket;	/* prime number buckets for Security bit in Catalog btree */
@@ -798,6 +803,9 @@ enum
 
 //#define S_LockedName			0x4000  // locked file name
 
+/* Journal status flag (contents of JStat) */
+#define S_BadJournal		0x8000	/* Bad journal content */
+
 /*------------------------------------------------------------------------------
  ScavCtrl Interface
 ------------------------------------------------------------------------------*/
@@ -853,8 +861,10 @@ enum {
 	M_Look		               	= 21,
 	M_OtherWriters		       	= 22,
 	M_CaseSensitive		       	= 23,
+	M_ReRepairFailed			= 24,			
+	M_LookDamagedDir			= 25,
 
-	M_LastMessage               = 23
+	M_LastMessage               = 25
 };
 
 /*------------------------------------------------------------------------------
@@ -882,7 +892,7 @@ enum {
 	E_ABlkSt		= -509,	/* Invalid allocation block start */
 
 	E_ExtEnt		= -510,	/* Invalid extent entry */
-	E_OvlExt		= -511,	/* overlapped extent allocation */
+	E_OvlExt		=  511,	/* overlapped extent allocation */
 	E_LenBTH		= -512,	/* Invalid BTH length */
 	E_ShortBTM		= -513,	/* BT map too short to repair */
 	E_BTRoot		= -514,	/* Invalid root node number */
@@ -949,11 +959,13 @@ enum {
 
 	E_InvalidUID		=  570,
 	E_IllegalName		=  571,
+	E_IncorrectNumThdRcd	=  572,
 	/* Init the next two errors to pre-existing messages.  Fix them in 3964748 */
 	E_IncorrectAttrCount	=  547,	/* Incorrect attributes in attr btree with attr bits in catalog btree */
 	E_IncorrectSecurityCount=  547, /* Incorrect security attributes in attr btree with security bits in catalog btree */
+	E_SymlinkCreate		=  573,
 
-	E_LastError		=  571
+	E_LastError		=  573
 };
 
 
@@ -1029,6 +1041,15 @@ extern	int		FixDFCorruption( const SGlobPtr GPtr, RepairOrderPtr DFOrderP );
 
 extern	OSErr	ProcessFileExtents( SGlobPtr GPtr, SFCB *fcb, UInt8 forkType, UInt16 flags, Boolean isExtentsBTree, Boolean *hasOverflowExtents, UInt32 *blocksUsed  );
 
+/* Function to get return file path/name given an ID */
+extern 	OSErr 	GetSystemFileName(UInt32 fileID, char *filename, unsigned int *filenamelen);
+extern 	OSErr 	GetFileNamePathByID(SGlobPtr GPtr, UInt32 fileID, char *fullPath, unsigned int *fullPathLen, char *fileName, unsigned int *fileNameLen, u_int16_t *status);
+#define FNAME_BUF2SMALL	0x001	/* filename buffer was too small */
+#define FNAME_BIGNAME	0x002	/* filename is greater than NAME_MAX bytes */
+#define FPATH_BUF2SMALL	0x010	/* path buffer was too small */
+#define	FPATH_BIGNAME	0x020	/* intermediate component in path is greater than NAME_MAX bytes */
+#define	F_RESERVE_FILEID 0x100	/* file ID was less than kHFSFirstUserCatalogNodeID */
+
 /* ------------------------------- From SUtils.c --------------------------------- */
 
 extern	int		AllocBTN( SGlobPtr GPtr, short FilRefN, UInt32 NodeNum );
@@ -1061,7 +1082,7 @@ extern Boolean 	VolumeObjectIsHFS( void );
 extern Boolean 	VolumeObjectIsEmbeddedHFSPlus( void );
 extern Boolean 	VolumeObjectIsPureHFSPlus( void );
 extern void 	RecordXAttrBits(SGlobPtr GPtr, UInt16 flags, HFSCatalogNodeID fileid, UInt16 btreetype); 
-extern int 	ComparePrimeBuckets(SGlobPtr GPtr, UInt16 BitMask); 
+extern int ComparePrimeBuckets(SGlobPtr GPtr, UInt16 BitMask); 
 
 extern	void	InvalidateCalculatedVolumeBitMap( SGlobPtr GPtr );
 
@@ -1089,6 +1110,7 @@ extern	OSErr	AttrBTChk( SGlobPtr GPtr );		//	attributes btree check
 extern	OSErr	IVChk( SGlobPtr GPtr );
 
 extern	int	CheckForClean( SGlobPtr GPtr, Boolean markClean );
+
 extern  int	CheckIfJournaled(SGlobPtr GPtr);
 
 extern	OSErr	VInfoChk( SGlobPtr GPtr );
@@ -1166,6 +1188,12 @@ void ExtDataRecToExtents(
 	const HFSExtentRecord	oldExtents,
 	HFSPlusExtentRecord	newExtents);
 
+OSErr UpdateExtentRecord (
+	const SVCB		*vcb,
+	SFCB					*fcb,
+	const HFSPlusExtentKey	*extentFileKey,
+	HFSPlusExtentRecord		extentData,
+	UInt32					extentBTreeHint);
 
 OSErr	CheckFileExtents( SGlobPtr GPtr, UInt32 fileNumber, UInt8 forkType,
                           const void *extents, UInt32 *blocksUsed );
@@ -1281,8 +1309,7 @@ EXTERN_API_C( SInt32 )
 CompareExtentKeysPlus			(const HFSPlusExtentKey * searchKey,
 								 const HFSPlusExtentKey * trialKey);
 EXTERN_API_C( SInt32 )
-CompareAttributeKeys			(const AttributeKey *	searchKey,
-					 			 const AttributeKey *	trialKey);
+CompareAttributeKeys			(const AttributeKey * searchKey,  const AttributeKey * trialKey);
 EXTERN_API( SFCB* )
 ResolveFCB						(short 					fileRefNum);
 
@@ -1381,8 +1408,8 @@ Ptr	GetFCBSPtr( void );
 /* 
  * UTF-8 conversion routines
  */
-extern int utf_decodestr(const u_int8_t *, size_t, u_int16_t *, size_t *);
-extern int utf_encodestr(const u_int16_t *, size_t, u_int8_t *, size_t *);
+extern int utf_decodestr(const unsigned char *, size_t, u_int16_t *, size_t *);
+extern int utf_encodestr(const u_int16_t *, size_t, unsigned char *, size_t *);
 
 /* 
  * HardLink checking routines
@@ -1400,7 +1427,7 @@ extern int  BitMapCheckEnd(void);
 extern int  CaptureBitmapBits(UInt32 startBit, UInt32 bitCount);
 extern int  ReleaseBitmapBits(UInt32 startBit, UInt32 bitCount);
 extern int  CheckVolumeBitMap(SGlobPtr g, Boolean repair);
-
+extern void UpdateFreeBlockCount(SGlobPtr g);
 
 #ifdef __cplusplus
 };
